/*
 * Segment 10 System (Sys), SWE 10.1 StartStopp
 * Copyright (C) 2007-2021 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:<br>
 * BitCtrl Systems GmbH<br>
 * Weißenfelser Straße 67<br>
 * 04229 Leipzig<br>
 * Phone: +49 341-490670<br>
 * mailto: info@bitctrl.de
 */

package de.bsvrz.sys.startstopp.console;

import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.concurrent.TimeUnit;

import com.google.common.base.Objects;

import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.startstopp.api.KeyStoreData;
import de.bsvrz.sys.startstopp.api.StartStoppClient;
import de.bsvrz.sys.startstopp.api.StartStoppException;
import de.bsvrz.sys.startstopp.api.jsonschema.Applikation;

/**
 * Hilfsklasse zum Starten oder Beenden einer StartStopp-Applikation von der
 * Kommandozeile.
 * 
 * Die Anwendung verbindet sich mit der gewünschten StartStopp-Applikation und
 * versendet eine Anforderung zum Starten oder Beenden einer Inkarnation.
 * 
 * Über folgende Aufrufparameter kann das Verhalten der Anwendung gesteuert
 * werden:
 * 
 * <ul>
 * <li>hostname=&lt;name&gt; bestimmt per Name oder IP-Adresse den Host der
 * StartStopp-Applikation, der Standardwert ist 'localhost'</li>
 * <li>port=&lt;port&gt; bestimmt den Port über den die StartStopp-Applikation
 * erreicht werden kann, der Standardwert ist '3000'</li>
 * <li>kommando=&lt;starten|beenden&gt; das Kommando, welches auf eine
 * Inkarnation ausgeführt werden soll</li>
 * <li>inkarnation=&lt;Name der Inkarnation&gt; der Name der Inkarnation auf
 * welche das Kommando ausgeführt werden soll.</li>
 * <li>timeoutInMinuten=&lt;wert&gt; bestimmt die Zeit, die maximal auf das Ende
 * des StartStopp-Servers gewartet wird in Minuten. Der Standardwert ist '0',
 * d.h. es wird unendlich lange gewartet</li>
 * </ul>
 * 
 * @author BitCtrl Systems GmbH, Christian Hösel
 */
public class StartStoppKommandoSender {

	private static final String BEENDEN = "beenden";
	private static final String STARTEN = "starten";
	private Debug logger;
	private String host;
	private int port;

	private KeyStoreData keyStoreData;
	private String inkarnation;
	private String kommando;
	private int timeoutInMinuten;

	public static void main(String[] args) {
		StartStoppKommandoSender stopper = new StartStoppKommandoSender(args);
		stopper.versendeKommando();
	}

	private StartStoppKommandoSender(String[] args) {
		keyStoreData = new KeyStoreData(args);
		ArgumentList argList = new ArgumentList(Arrays.copyOf(args, args.length));

		Debug.init("StartStopp", argList);
		logger = Debug.getLogger();

		host = argList.fetchArgument("-host=localhost").asString();
		port = Integer.parseInt(argList.fetchArgument("-port=3000").asString());

		inkarnation = argList.fetchArgument("-inkarnation=").asString();
		kommando = argList.fetchArgument("-kommando=").asString();
		timeoutInMinuten = Integer.parseInt(argList.fetchArgument("-timeoutInMinuten=0").asString());

		logger.info("Beende StartStopp auf Host: " + host + " Port: " + port);
	}

	private void versendeKommando() {

		StartStoppClient client = new StartStoppClient(host, port, keyStoreData);
		try {
			if (Objects.equal(STARTEN, kommando)) {
				client.starteApplikation(inkarnation);
				warteAufInkarantionsZustand(client);
			} else if (Objects.equal(BEENDEN, kommando)) {
				client.stoppeApplikation(inkarnation);
				warteAufInkarantionsZustand(client);
			}

		} catch (StartStoppException e) {
			logger.error(
					"Anforderung " + kommando + " für Inkarnation " + inkarnation + " konnte nicht versendet werden",
					e);
			return;
		}
	}

	private void warteAufInkarantionsZustand(StartStoppClient client) {

		boolean run = true;
		LocalDateTime startZeit = LocalDateTime.now();
		while (run) {
			try {
				Applikation applikation = client.getApplikation(inkarnation);
				if (Objects.equal(STARTEN, kommando)
						&& Objects.equal(applikation.getStatus(), Applikation.Status.INITIALISIERT)) {
					return;
				} else if (Objects.equal(BEENDEN, kommando)
						&& Objects.equal(applikation.getStatus(), Applikation.Status.GESTOPPT)) {
					return;
				}
				if (timeoutInMinuten > 0) {
					if (LocalDateTime.now().isAfter(startZeit.plusMinutes(timeoutInMinuten))) {
						run = false;
						logger.warning("Inkarnation "+inkarnation+" steht im Status: " + applikation.getStatus()
								+ " und wurde nach " + timeoutInMinuten
								+ " Minuten immer noch nicht beendet, Vorgang wird abgebrochen.");
					}
				}
			} catch (StartStoppException e) {
				logger.fine(e.getLocalizedMessage());
				run = false;
				logger.info("StartStopp-Aplikation ist nicht mehr erreichbar und wird als beendet betrachtet!");
			}
			try {
				TimeUnit.SECONDS.sleep(1);
			} catch (InterruptedException e) {
				logger.error(e.getLocalizedMessage());
			}
		}

	}
}
